#!/bin/bash


#Preupgrade assistant performs system upgradability assessment
#and gathers information required for successful operating system upgrade.
#Copyright (C) 2013 Red Hat Inc.
#Frantisek Kluknavsky <fkluknav@redhat.com>
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.
. /usr/share/preupgrade/common.sh
check_applies_to ""
COMPONENT=""
#END GENERATED SECTION

if [ ! -f "/etc/redhat-release" ]; then
    log_error "Could not determine version. Is this really Red Hat Enterprise Linux 6?"
    exit_error
fi
grep "Red Hat Enterprise Linux Server release 6." /etc/redhat-release
if [ $? -ne 0 ]; then
    log extreme_risk "This system is $(cat /etc/redhat-release)"
    log_extreme_risk "Only upgrade of Red Hat Enterprise Linux 6 Server is supported."
    exit_fail
fi

exit_pass

