#! /usr/bin/env bash

KRB_SYSCONFIG_FILE=krb5kdc

# if the modified config file exists, copy it to the system, set its owner, access rights and restore its context
if test -s "$KRB_SYSCONFIG_FILE"; then
	cp "$KRB_SYSCONFIG_FILE" /etc/sysconfig/krb5kdc
	chown root:root /etc/sysconfig/krb5kdc
	chmod 644 /etc/sysconfig/krb5kdc
	restorecon /etc/sysconfig/krb5kdc
fi

exit 0
