#!/bin/bash

# This postcript solves rhbz#1078222

PKG="selinux-policy-sandbox"

yum install --nogpgcheck --enablerepo="redhat-upgrade-*" -y ${PKG}

INST_IS_OK=${?}

if [[ ${INST_IS_OK} -ne 0 ]]
then
    echo "installation of ${PKG} failed"
fi

exit ${INST_IS_OK}
