#!/bin/bash


#Preupgrade assistant performs system upgradability assessment
#and gathers information required for successful operating system upgrade.
#Copyright (C) 2013 Red Hat Inc.
#Lukáš Nykrýn <lnykryn@redhat.com>
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.
. /usr/share/preupgrade/common.sh
check_applies_to "initscripts"
COMPONENT="initscripts"
#END GENERATED SECTION

LIST=""

for i in /etc/sysconfig/network-scripts/ifcfg-*; do
        unset DEVICE
        unset HWADDR
        . $i

        if [[ $DEVICE =~ eth[0-9]+ && -n  $HWADDR ]]; then
                LIST="$HWADDR $LIST"
        fi
done

LIST=$(printf '%s\n' $LIST | sort -u)

if [ $(echo $LIST | wc -w) -ge 2 ]; then
        log_high_risk initscripts "Your ifcfg files are assigning fixed 'ethX' names to network cards."
        exit_fail
fi

exit_pass
